#uses "fwInstallationManager"

/**@name LIBRARY: pcSystemIntegrity_GW_FGC.ctl

@author: Herve Milcent (AB-CO)

Creation Date: 07/05/2007

Modification History: 
		
version 1.0
  24/06/2014: Jean-Charles 
  -ENS-10480 Remove any reference to the dpe communication.counter

  15/11/2011: Herve
  - IS-651     system name and component version added in the systemIntegrity diagnostic 
 
  16/06/2011: Herve
  - IS-562: get the list of systemAlarm pattern

External Functions: 
	
Internal Functions: 
	
Purpose: 
	Library of the GW_FGC component used by the systemIntegrity.
	 	
Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. global variable: the following variables are global to the script
	. data point type needed: _UnSystemIntegrity
	. data point: GW_FGC_systemIntegrity
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

// global declaration
global dyn_string g_pcSystemIntegrity_GW_FGC_DSList; // list of the GW_FGC-DS dp checked
global dyn_int g_pcSystemIntegrity_GW_FGC_DSThreadId; // list of the GW_FGC-DS thread Id checking the counter
global int g_pcSystemIntegrity_GW_FGCCheckingDelay;
global bool g_bGW_FGCNotStarted=true;
mapping g_mpcSystemIntegrity_TimeAlarm; // variable used to keep the latest alarm.

//@{

//------------------------------------------------------------------------------------------------------------------------
// GW_FGC_systemIntegrityInfo
/** Return the list of systemAlarm pattern. 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@return list of systemAlarm pattern
*/
dyn_string GW_FGC_systemIntegrityInfo()
{
  return makeDynString(DS_pattern);
}

//------------------------------------------------------------------------------------------------------------------------

// pcSystemIntegrity_GW_FGC_Initialize
/**
Purpose:
Get the list of GW_FGC check by the systemIntegrity. The ones that are enabled.

@param dsResult: dyn_string, output, the list of enabled GW_FGC that are checked

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
pcSystemIntegrity_GW_FGC_Initialize(dyn_string &dsResult)
{
	dyn_string dsList;
	int len, i;
	string dpToCheck;
	bool enabled;

	dpGet(UN_SYSTEM_INTEGRITY_GW_FGC+UN_SYSTEMINTEGRITY_EXTENSION+".config.data", dsList);
	pcSystemIntegrity_GW_FGC_DataCallback("", dsList);

	dsList = dpNames(c_unSystemAlarm_dpPattern+DS_pattern+"*", c_unSystemAlarm_dpType);
	len = dynlen(dsList);
	for(i = 1; i<=len; i++) {
		dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
		dpToCheck = substr(dpToCheck, strlen(c_unSystemAlarm_dpPattern+DS_pattern),strlen(dpToCheck));
		if(dpExists(dpToCheck)) {
			if(dpTypeName(dpToCheck) == PC_GW_FGC_DPTYPE) {
				dpGet(dsList[i]+".enabled", enabled);
				if(enabled)
					dynAppend(dsResult, dpToCheck);
			}
		}
	}
//DebugN("GW_FGC_DS:", dsList, dsResult);
}

//------------------------------------------------------------------------------------------------------------------------

// pcSystemIntegrity_GW_FGC_DataCallback
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param sConfigData: string, input, the config data

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
pcSystemIntegrity_GW_FGC_DataCallback(string sDpe1, dyn_string dsConfigData)
{
	dyn_string dsTemp=dsConfigData;

	while(dynlen(dsTemp) <=2)
		dynAppend(dsTemp, "0");

	g_pcSystemIntegrity_GW_FGCCheckingDelay = dsTemp[1];
	if(g_pcSystemIntegrity_GW_FGCCheckingDelay <= 0)
		g_pcSystemIntegrity_GW_FGCCheckingDelay = c_pcSystemIntegrity_defaultGW_FGCCheckingDelay;

//DebugN(dsConfigData, g_pcSystemIntegrity_GW_FGCCheckingDelay);
}

//------------------------------------------------------------------------------------------------------------------------

// pcSystemIntegrity_GW_FGC_HandleCommand
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
pcSystemIntegrity_GW_FGC_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len =dynlen(parameters);
  string sMessage;
	
	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				pcSystemIntegrity_GW_FGC_checking(parameters[i], true, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				pcSystemIntegrity_GW_FGC_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]);				
			}
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
					pcSystemIntegrity_GW_FGC_checking(parameters[i], false, true, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
					pcSystemIntegrity_GW_FGC_checking(parameters[i], false, false, exceptionInfo);
			}
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSet(UN_SYSTEM_INTEGRITY_GW_FGC+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
					UN_SYSTEM_INTEGRITY_GW_FGC+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_pcSystemIntegrity_GW_FGC_DSList);
			break;
    case UN_SYSTEMINTEGRITY_VERSION:
      sMessage = unGenericDpFunctions_getComponentsVersion(makeDynString("pcDevices"));
      dpSet(UN_SYSTEM_INTEGRITY_GW_FGC+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
            UN_SYSTEM_INTEGRITY_GW_FGC+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", makeDynString(getCurrentTime(),sMessage));
      break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
			unMessageText_sendException("*", "*", "pcSystemIntegrity_GW_FGC_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error uin case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugN(getCurrentTime(), exceptionInfo);
		}
	}

// do here the dpConnect to the list of drivers
	if(g_bGW_FGCNotStarted) {
		dpConnect("pcSystemIntegrity_GW_FGC_ConnectionDriverCB", CONNECTION_DRIVER);
		g_bGW_FGCNotStarted = false;
	}

//DebugN(sDpe1, command, parameters, g_pcSystemIntegrity_GW_FGC_DSList);
}

//------------------------------------------------------------------------------------------------------------------------

// pcSystemIntegrity_GW_FGC_ConnectionDriverCB
/**
Purpose:
callback on the list of connected driver.

@param sDpe1: string, input, dpe
@param diDriverNum: dyn_int, input, the list of driver number started

Usage: Public

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
pcSystemIntegrity_GW_FGC_ConnectionDriverCB(string sDp, dyn_int diDriverNum)
{
	dyn_string dsList = g_pcSystemIntegrity_GW_FGC_DSList;
	string sDpName;
	int i, len;
  
  //JCT - Nov 2017: Legacy code, not sure what/why it is here
	len = dynlen(dsList);
	for(i=1;i<=len;i++) 
  {
		sDpName = substr(dsList[i], strlen(c_unSystemAlarm_dpPattern+DS_pattern), strlen(dsList[i]));
		unSystemIntegrity_CMW_ConnectionDriver(sDpName, diDriverNum);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcSystemIntegrity_GW_FGC_checking
/**
Purpose:
This function register/de-register the callback funtion of GW_FGC manager. This function can also create the _unSystemAlarm_GW_FGC dp 
with the alarm config but it cannot delete it.

	@param sDp: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bRegister: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

pcSystemIntegrity_GW_FGC_checking(string sDp, bool bCreate, bool bRegister, dyn_string &exceptionInfo)
{
	string sDpToChek, dp, dpError, dpPlc, plcHostname, description;
	int res, thId, pos, drvNum, len,i;
	bool bError = false;
	dyn_string plc_plc_list;

	string alertPanel,alertHelp;
	bool configExists, isActive;
	int alertConfigType;
	dyn_string alertTexts,alertClasses,summaryDpeList,alertPanelParameters;
	dyn_float alertLimits;
	
// remove the system name
	res = strpos(sDp, ":");
	if(res >= 0)
		dp = substr(sDp, res+1, strlen(sDp));
	else
		dp = sDp;
	sDpToChek = dp;
	dp = c_unSystemAlarm_dpPattern+DS_pattern+dp;

// in case the Dp is not of type _UnPlc do nothing. Should not happen, just for security.
	if(dpTypeName(sDpToChek) != PC_GW_FGC_DPTYPE)
		return;
		
	dpGet(sDpToChek+".communication.driver_num", drvNum);

	if(bCreate) 
  {
    // get the plc hostname and the driver number
		plcHostname = unGenericDpFunctions_getAlias(sDpToChek);
    // create the sDp and its alarm config if it is not existing
		if(!dpExists(dp)) 
    {
			description = getCatStr("unSystemIntegrity", "PLC_DS_DESCRIPTION")+plcHostname+" -> DS driver "+ drvNum;
			unSystemIntegrity_createSystemAlarm(sDpToChek, DS_pattern, description, exceptionInfo);
		}
	}

	if(bRegister) 
  {
    // connect to the callback function if not already done
		pos = _unSystemIntegrity_isInList(g_pcSystemIntegrity_GW_FGC_DSList, dp);
		if(pos <= 0) 
    {
	    // add it in the list, because one of the callback function needs it.				
			pos = _unSystemIntegrity_setList(g_pcSystemIntegrity_GW_FGC_DSList, dp, true);

      // start the thread for checking that the driver is running
      dpConnectUserData(UN_SYSTEM_INTEGRITY_GW_FGC_check, makeDynMixed(drvNum, sDpToChek), CONNECTION_DRIVER);
      
   
			// set the enable to true and activate the alarm if it is not activated.
				dpSet(dp+".enabled", true);
				unAlarmConfig_mask(dp+".alarm", false, exceptionInfo);
			// get all the PLC-PLC dp, and enable them
				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+PLC_PLC_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
				len = dynlen(plc_plc_list);
				for(i=1;i<=len;i++) {
					dpSet(plc_plc_list[i]+".enabled", true);
					unAlarmConfig_mask(plc_plc_list[i]+".alarm", false, exceptionInfo);
				}

			// get all the FESystemAlarm dp, and enable them, check the Ok state and set the correct value
				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+FE_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
				len = dynlen(plc_plc_list);
				for(i=1;i<=len;i++) {
					dpSet(plc_plc_list[i]+".enabled", true);
					unAlarmConfig_mask(plc_plc_list[i]+".alarm", false, exceptionInfo);
				}
		}
	}
	else {
// disconnect the callback function
//DebugN("Disable", g_pcSystemIntegrity_GW_FGC_DSThreadId, "end");
		pos = _unSystemIntegrity_isInList(g_pcSystemIntegrity_GW_FGC_DSList, dp);
		if(pos > 0) {
    // kill the thread
      dpDisconnectUserData(UN_SYSTEM_INTEGRITY_GW_FGC_check, makeDynMixed(drvNum, sDpToChek), CONNECTION_DRIVER);
			//res = stopThread(g_pcSystemIntegrity_GW_FGC_DSThreadId[pos]);



    			// remove from list
    				pos = _unSystemIntegrity_setList(g_pcSystemIntegrity_GW_FGC_DSList, dp, false);
      //DebugN("rmv", res);
  			// set the enable to false and de-activate the alarm and acknowledge it if necessary
  				dpSet(dp+".enabled", false, dp+".alarm", c_unSystemIntegrity_no_alarm_value);
  				unAlarmConfig_mask(dp+".alarm", true, exceptionInfo);
  			// get all the PLC-PLC dp, and disable them
  				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+PLC_PLC_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
  				len = dynlen(plc_plc_list);
  				for(i=1;i<=len;i++) {
        // do not reset the alarm value because this DPE should be connected to an address config
  					dpSet(plc_plc_list[i]+".enabled", false);
  					unAlarmConfig_mask(plc_plc_list[i]+".alarm", true, exceptionInfo);
  				}

  			// get all the FESystemAlarm dp, and disable them, check the Ok state and set the correct value
  				plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+FE_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
  				len = dynlen(plc_plc_list);
  				for(i=1;i<=len;i++) {
        // do not reset the alarm value because this DPE should be connected to an address config
  					dpSet(plc_plc_list[i]+".enabled", false);
  					unAlarmConfig_mask(plc_plc_list[i]+".alarm", true, exceptionInfo);
  				}
   
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcSystemIntegrity_GW_FGCCheck
/**
Purpose:
	@param sGW_FGCDp: string, input, data point name

Usage: Internal

PVSS manager usage: CTRL (PVSS00CTRL)

dyn_mixed dmObj: contains at [1] the driver number to check and at [2] the name of the FrontEnd

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
pcSystemIntegrity_GW_FGCCheck(dyn_mixed dmObj, string sDp, dyn_int diDriverNum)
{

  //We expect exactly 2 items, c.f. function description
  if ( dynlen(dmObj) != 2) 
  {
    DebugTN(__FUNCTION__, "Error", "The paramater dmObj should contain 2 items and "+dynlen(dmObj+" instead!", dmObj));
    return;
  }  
  
  //Simply check if the FE driver is in the list of running drivers
  int alarmValue = dynContains( diDriverNum, dmObj[1]) > 0 ? c_unSystemIntegrity_no_alarm_value : 10;
  dpSetWait(c_unSystemAlarm_dpPattern+DS_pattern+dmObj[2]+".alarm", alarmValue);
}


//------------------------------------------------------------------------------------------------------------------------

//@}

